/*
 * Copyright (c) 2021 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
define(["require", "exports", "system_lib/Metadata", "system_lib/Feed", "system/SimpleHTTP"], function (require, exports, Metadata_1, feed, SimpleHTTP_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Maas = void 0;
    //  API GraphQL query endpoint URL for requesting data fmr database
    var kQueryUrl = "https://api.maas.museum/graphql?query=";
    /** Main class establishing the feed instances s provided by me. I just pick a number
        of interesting collections, giving them a short name that can be referred to
        from within Blocks.
    */
    var Maas = /** @class */ (function (_super) {
        __extends(Maas, _super);
        function Maas(env) {
            var _this = _super.call(this, env) || this;
            _this.establishFeed(new Collection("Matchbox", "Matchbox Toys")); // ***
            _this.establishFeed(new Collection("Clyde", "Clyde Engineering Photograph"));
            _this.establishFeed(new Collection("Observatory", "Sydney Observatory"));
            _this.establishFeed(new Collection("Wunderlich", "Wunderlich Limited Archive"));
            _this.establishFeed(new Collection("Stewart", "A A Stewart Engineering Model")); // ***
            _this.establishFeed(new Collection("Lennon", "Tom Lennon Photographic Archive"));
            return _this;
        }
        return Maas;
    }(feed.Feed));
    exports.Maas = Maas;
    /**
     * Manage a specific "collection" as its own feed instance, making it easy for
     * different display spots to show different collections. I just pick the first
     * objects in each collection as they exist in the database. In a real-world
     * use case, the database should contain some flag or priority value or other
     * criteria that can be selected for and sorted on for the desired set of
     * objects to appear on screen.
     */
    var Collection = /** @class */ (function () {
        function Collection(feedName, collectionName) {
            // Mandatory fields describing what types of list and item data I provide
            this.listType = ListItem;
            this.itemType = DetailsItem;
            this.name = feedName;
            this.collectionName = collectionName;
        }
        /**
         * Mandatory method used to get a list of items.
         */
        Collection.prototype.getList = function (spec) {
            /*	Define the GraphQL query to send to get a reasonably sized list of items.
                The request is designed to return a subset of the fields in each item,
                only what's needed to display the list, including a thumbnail version
                of its first image.
            */
            var graphQl = "{\n\t\t\tobjects(limit: 30, skip: 0, filter: {\n\t\t\t\tcollectionName:  \"" + this.collectionName + "\",\n\t\t\t\thasMedia:true\n\t\t\t}) {\n\t\t\t\t_id\n\t\t\t\tdisplayTitle\n\t\t\t\timages(limit: 1) {\n\t\t\t\t\tthumb: url(width: 250, height: 250, smart:true)\n\t\t\t\t}\n\t\t\t}\n\t\t}";
            // Use SimpleHTTP to fetch the list from the server, passing it to processList
            return SimpleHTTP_1.SimpleHTTP.newRequest(kQueryUrl + encodeURI(graphQl), { interpretResponse: true })
                .setTimeout(25) // Yes, it occasionally takes quite a while to respond
                .get()
                .then(function (response) { return Collection.processList(response.interpreted.data.objects); });
        };
        /**
         * Convert list of items returned from GraphQL API to our desired ListData form.
         */
        Collection.processList = function (list) {
            var listItems = [];
            for (var _i = 0, list_1 = list; _i < list_1.length; _i++) {
                var item = list_1[_i];
                // Pass explicit null if has no thumbnail to not request through details
                listItems.push(new ListItem(item._id, item.displayTitle, item.statement, Collection.getThumbUrl(item)));
            }
            return { items: listItems };
        };
        /** Mandatory method used to get a single item with full details, based
            on the ID requested according to spec.
         */
        Collection.prototype.getDetails = function (spec) {
            var graphQl = "{\n\t\t\tobject(filter: {\n\t\t\t\t_id: " + spec.id + "\n\t\t\t}) {\n\t\t\t\t_id\n\t\t\t\tdisplayTitle\n\t\t\t\tdescription\n\t\t\t\tstatement\n\t\t\t\timages(limit: 6) {\n\t\t\t\t  large: url(width: 1920)\n\t\t\t\t  thumb: url(width: 250, height: 250, smart:true)\n\t\t\t\t}\n\t\t\t}\n\t\t}";
            return SimpleHTTP_1.SimpleHTTP.newRequest(kQueryUrl + encodeURI(graphQl), { interpretResponse: true })
                .setTimeout(25) // Yes, it occasionally takes quite a while to respond
                .get()
                .then(function (response) { return Collection.processDetails(response.interpreted.data.object); });
        };
        /**
         * Return the first thumbnail (if any) in the images array in item.
         * Return explicit null if has no thumbnail to differentiate from
         * undefined which is in places interpreted as "not available"
         */
        Collection.getThumbUrl = function (item) {
            var firstImage = item.images[0];
            return firstImage ? firstImage.thumb : null;
        };
        /**
         * Convert single item returned from GraphQL API to our desired DetailsItem.
         */
        Collection.processDetails = function (item) {
            var images = [];
            for (var _i = 0, _a = item.images; _i < _a.length; _i++) {
                var image = _a[_i];
                images.push(new Image(image.large, image.thumb));
            }
            return new DetailsItem(item._id, item.displayTitle, item.statement, Collection.getThumbUrl(item), item.description, images);
        };
        return Collection;
    }());
    /**	Shape of each item in the brief list of items returned for
        display in the grid. The amount of data provided here is
        deliberately kept to a bare minimum, since we may need to deal
        with many objects of this type.
    */
    var ListItem = /** @class */ (function () {
        function ListItem(id, title, brief, thumb) {
            this.id = id;
            this.title = title;
            this.thumb = thumb;
        }
        __decorate([
            Metadata_1.id("Uniquely identifying this feed item"),
            __metadata("design:type", Number)
        ], ListItem.prototype, "id", void 0);
        __decorate([
            Metadata_1.field("Title of the item"),
            __metadata("design:type", String)
        ], ListItem.prototype, "title", void 0);
        __decorate([
            Metadata_1.field("Small (250x250) thumbnail image URL"),
            __metadata("design:type", String)
        ], ListItem.prototype, "thumb", void 0);
        return ListItem;
    }());
    /**	Shape of a single image, used in DetailsItem since it can provide a
        list of images associated with that single item
    */
    var Image = /** @class */ (function () {
        function Image(large, thumb) {
            this.large = large;
            this.thumb = thumb;
        }
        __decorate([
            Metadata_1.field("URL to large (1080) image"),
            __metadata("design:type", String)
        ], Image.prototype, "large", void 0);
        __decorate([
            Metadata_1.field("Small (250x250) thumbnail image URL"),
            __metadata("design:type", String)
        ], Image.prototype, "thumb", void 0);
        return Image;
    }());
    /**	A full details item, fetched to view all desired details about a single
        item, possibly including lists of nested sub-items, such as the Image field.
        Note that the DetailsItem must extend the ListItem, and hence must provide
        that same data, plus its own extended data.
    */
    var DetailsItem = /** @class */ (function (_super) {
        __extends(DetailsItem, _super);
        function DetailsItem(id, title, brief, thumb, details, images) {
            var _this = this;
            _this = _super.call(this, id, title, brief, thumb) || this,
                _this.brief = brief;
            _this.details = details;
            _this.images = images;
            return _this;
        }
        __decorate([
            Metadata_1.field("Detailed description"),
            __metadata("design:type", String)
        ], DetailsItem.prototype, "details", void 0);
        __decorate([
            Metadata_1.field("Brief description"),
            __metadata("design:type", String)
        ], DetailsItem.prototype, "brief", void 0);
        __decorate([
            Metadata_1.list(Image, "All images associated with this item"),
            __metadata("design:type", Array)
        ], DetailsItem.prototype, "images", void 0);
        return DetailsItem;
    }(ListItem));
});
