/*
 * Copyright (c) 2021 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
 * A minimalistic example of a feedscript that lists files in a directory and expose those as a feed to Blocks.
 * under the /public/ or /temp/ directory in the blocks servers PIXILAB-Blocks-root directory.
 */

import { id } from '../system_lib/Metadata';
import { SimpleFile } from '../system/SimpleFile';
import { Feed, FeedDetailsSpec, FeedEnv, FeedListSpec, StaticFeed } from '../system_lib/Feed';

export class SimpleSlideshowFeed extends Feed {

	constructor(env: FeedEnv) {
		super(env);
		this.establishFeed(new SlideshowFeed('Images','Image'));
	}
}

class SlideshowFeed implements StaticFeed<Data,Data> {
	readonly name: string;
	//readonly index: number;
	readonly slidesPath: string;

	listType = Data;
	itemType = Data;

	private slideshowImageList: Data[];

	constructor(
		name: string,
		nameDetails: string
	) {
		this.name = name;
		this.slidesPath = '/public/feed/slideshow/'; //Edit this line to change path to the slide images.
	}

	async getList(spec: FeedListSpec) {
		const slideshowImageListRaw = await SimpleFile.list(this.slidesPath,true);
		const slideshowImageList = Feed.makeJSArray(slideshowImageListRaw.files);
		this.slideshowImageList = slideshowImageList.map(imageFileName => new Data(this.slidesPath + imageFileName));
		return { items: this.slideshowImageList };
	}

	async getDetails(spec: FeedDetailsSpec) {
		return new Data('test.jpg');  //There are no details but we got something just to satisfy Blocks
	}
}

class Data {

	@id("File URL")
	readonly url: string;

	constructor(
		url: string
	) {
		this.url = url;
	}
}
