/*
 * Copyright (c) 2021 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
 */

import { field, id } from '../system_lib/Metadata';
import { SimpleFile } from '../system/SimpleFile';
import { Feed, FeedDetailsSpec, FeedEnv, FeedListSpec, StaticFeed } from '../system_lib/Feed';

export class SlideshowList extends Feed {

	constructor(env: FeedEnv) {
		super(env);
		this.establishFeed(
			new SlideshowListFeed(
				'SlideshowListFeed'
			)
		);

	}
}

class SlideshowListFeed implements StaticFeed<Data, Data> {
	readonly name: string;
	listType = Data;
	itemType = Data;

	private slideshowList: Data[];

	constructor(
		name: string
	) {
		this.name = name;
	}

	async getList(spec: FeedListSpec) {
		const slideshowListRaw = await SimpleFile.list('/public/feed/slideshow');
		const slideshowList = Feed.makeJSArray(slideshowListRaw.directories);
		this.slideshowList = slideshowList.map((slideShowDirPath, i) => {
			let lastSlashPos = slideShowDirPath.lastIndexOf('/');
			const slideshowName = slideShowDirPath.substring(lastSlashPos + 1, slideShowDirPath.length);

			return new Data(
				i,
				slideshowName
			)
		});

		return { items: this.slideshowList };
	}

	async getDetails(spec: FeedDetailsSpec) {
		return new Data(0, 'test');
	}
}

class Data {

	@id("Index")
	readonly index: number;

	@id("Slideshow name")
	readonly name: string;

	constructor(
		index: number,
		name: string
	) {
		this.index = index;
		this.name = name;
	}
}
