/*
 * Copyright (c) 2021 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
 * A simple example of a feedscript that finds any slideshow directory stored in the selected slideshow path stored
 * under the /public/ or /temp/ directory in the blocks servers PIXILAB-Blocks-root directory.
 */

import { id } from '../system_lib/Metadata';
import { SimpleFile } from '../system/SimpleFile';
import { Feed, FeedDetailsSpec, FeedEnv, FeedListSpec, StaticFeed } from '../system_lib/Feed';

export class Slideshow extends Feed {

	constructor(env: FeedEnv) {
		super(env);
		this.getSlideshowDirectories();
	}

	async getSlideshowDirectories() {
		const slideshowListRaw = await SimpleFile.list('/public/feed/slideshows',false); //List the files and directories as usable path in the prefered slideshows directory.
		const slideshowList = Feed.makeJSArray(slideshowListRaw.directories);  //Make a string array out of any derictories found
		for (var i in slideshowList) {
			this.establishFeed(new SlideshowFeed(slideshowList[i], parseInt(i))); //Create one Slideshowfeed per directory found in path.
		}
	};
}

class SlideshowFeed implements StaticFeed<Data, Data> {
	readonly name: string;
	readonly index: number;
	readonly path: string;

	listType = Data;
	itemType = Data;

	private slideshowImageList: Data[];

	constructor(
		name: string,
		index: number
	) {
		this.path = name;
		this.name = SlideshowFeed.parseName(name);
		this.index = index;
		console.log(this.path);
	}

	async getList(spec: FeedListSpec) {
		const slideshowImageListRaw = await SimpleFile.list(this.path);
		const slideshowImageList = Feed.makeJSArray(slideshowImageListRaw.files); //
		this.slideshowImageList = slideshowImageList.map(imageURL => new Data(imageURL));
		return { items: this.slideshowImageList };
	}

	async getDetails(spec: FeedDetailsSpec) {
		return new Data('test.jpg');  //Görs detta för att tillgodose någon förutfattad mening hos Blocks?
	}

	private static parseName(slidesPath:string) {

			let lastSlashPos = slidesPath.lastIndexOf('\\');  //Only for windows!!?? Man skulle kunna kolla riktning första slash för att avgöra vad den skall vara.
			const slideshowName = slidesPath.substring(lastSlashPos + 1, slidesPath.length);
			return slideshowName;
	}


}

class Data {

	@id("File URL")
	readonly url: string;

	constructor(
		url: string
	) {
		this.url = url;
	}
}
